@@ -1,8 +1,8 @@
-/* $Id: global.c,v 1.72 2004/11/17 23:17:05 jordi Exp $ */
+/* $Id: global.c,v 1.90 2005/03/17 19:10:29 dolorous Exp $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
- *   Copyright (C) 1999-2004 Chris Allegretta                             *
+ *   Copyright (C) 1999-2005 Chris Allegretta                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -40,9 +40,9 @@ char *last_search = NULL;	/* Last string we searched for */
 char *last_replace = NULL;	/* Last replacement string */
 int search_last_line;		/* Is this the last search line? */
 
-long flags = 0;			/* Our flag containing many options */
+unsigned long flags = 0;	/* Our flag containing many options */
+WINDOW *topwin;			/* Top buffer */
 WINDOW *edit;			/* The file portion of the editor */
-WINDOW *topwin;			/* Top line of screen */
 WINDOW *bottomwin;		/* Bottom buffer */
 char *filename = NULL;		/* Name of the file */
 
@@ -63,17 +63,22 @@ filestruct *edittop = NULL;	/* Pointer to the top of the edit
 				   file struct */
 filestruct *filebot = NULL;	/* Last node in the file struct */
 filestruct *cutbuffer = NULL;	/* A place to store cut text */
+#ifndef DISABLE_JUSTIFY
+filestruct *jusbuffer = NULL;	/* A place to store unjustified text */
+#endif
 partition *filepart = NULL;	/* A place to store a portion of the
 				   file struct */
 
 #ifdef ENABLE_MULTIBUFFER
-openfilestruct *open_files = NULL;	/* The list of open files */
+openfilestruct *open_files = NULL;	/* The list of open file
+					   buffers */
 #endif
 
 #if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
 char *whitespace = NULL;	/* Characters used when displaying
 				   the first characters of tabs and
 				   spaces. */
+int whitespace_len[2];		/* The length of the characters. */
 #endif
 
 #ifndef DISABLE_JUSTIFY
@@ -99,7 +104,8 @@ char *backup_dir = NULL;	/* Backup directory. */
 
 char *answer = NULL;		/* Answer str to many questions */
 int totlines = 0;		/* Total number of lines in the file */
-long totsize = 0;		/* Total number of bytes in the file */
+size_t totsize = 0;		/* Total number of characters in the
+				   file */
 size_t placewewant = 0;		/* The column we'd like the cursor
 				   to jump to when we go to the
 				   next or previous line */
@@ -155,9 +161,7 @@ syntaxtype *syntaxes = NULL;
 char *syntaxstr = NULL;
 #endif
 
-#if !defined(DISABLE_BROWSER) || !defined(DISABLE_HELP) || !defined(DISABLE_MOUSE)
 const shortcut *currshortcut;	/* Current shortcut list we're using */
-#endif
 
 #ifndef NANO_SMALL
 toggle *toggles = NULL;
@@ -180,9 +184,7 @@ bool curses_ended = FALSE;	/* Indicates to statusbar() to simply
 				 * write to stderr, since endwin() has
 				 * ended curses mode. */
 
-#ifdef ENABLE_NANORC
 char *homedir = NULL;		/* $HOME or from /etc/passwd. */
-#endif
 
 size_t length_of_list(const shortcut *s)
 {
@@ -198,7 +200,7 @@ void sc_init_one(shortcut **shortcutage, int ctrlval, const char *desc,
 #ifndef DISABLE_HELP
 	const char *help,
 #endif
-	int metaval, int funcval, int miscval, int view, void
+	int metaval, int funcval, int miscval, bool view, void
 	(*func)(void))
 {
     shortcut *s;
@@ -237,6 +239,9 @@ void shortcut_init(bool unjustify)
     const char *cancel_msg = N_("Cancel");
     const char *first_line_msg = N_("First Line");
     const char *last_line_msg = N_("Last Line");
+#ifndef NANO_SMALL
+    const char *cut_till_end_msg = N_("CutTillEnd");
+#endif
 #ifndef DISABLE_JUSTIFY
     const char *beg_of_par_msg = N_("Beg of Par");
     const char *end_of_par_msg = N_("End of Par");
@@ -314,6 +319,10 @@ void shortcut_init(bool unjustify)
     const char *nano_opennext_msg = N_("Switch to the next file buffer");
 #endif
     const char *nano_verbatim_msg = N_("Insert character(s) verbatim");
+#ifndef NANO_SMALL
+    const char *nano_cut_till_end_msg =
+	N_("Cut from the cursor position to the end of the file");
+#endif
 #ifndef DISABLE_JUSTIFY
     const char *nano_fulljustify_msg = N_("Justify the entire file");
 #endif
@@ -348,7 +357,7 @@ void shortcut_init(bool unjustify)
     const char *nano_backup_msg = N_("Back up original file when saving");
     const char *nano_execute_msg = N_("Execute external command");
 #endif
-#if defined(ENABLE_MULTIBUFFER) && !defined(NANO_SMALL)
+#if !defined(NANO_SMALL) && defined(ENABLE_MULTIBUFFER)
     const char *nano_multibuffer_msg = N_("Insert into new buffer");
 #endif
 #ifndef DISABLE_BROWSER
@@ -379,19 +388,14 @@ void shortcut_init(bool unjustify)
 #endif
 		);
 
+    /* Translators: try to keep this string under 10 characters long */
+    sc_init_one(&main_list, NANO_EXIT_KEY,
 #ifdef ENABLE_MULTIBUFFER
-    if (open_files != NULL && (open_files->prev != NULL ||
-	open_files->next != NULL))
-    /* Translators: try to keep this string under 10 characters long */
-	sc_init_one(&main_list, NANO_EXIT_KEY, N_("Close"),
-		IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
-		NANO_NO_KEY, VIEW, do_exit);
-    else
+	open_files != NULL && open_files != open_files->next ?
+	N_("Close") :
 #endif
-    /* Translators: try to keep this string under 10 characters long */
-	sc_init_one(&main_list, NANO_EXIT_KEY, exit_msg,
-		IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
-		NANO_NO_KEY, VIEW, do_exit);
+	exit_msg, IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
+	NANO_NO_KEY, VIEW, do_exit);
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_WRITEOUT_KEY, N_("WriteOut"),
@@ -450,7 +454,7 @@ void shortcut_init(bool unjustify)
     /* Translators: try to keep this string under 10 characters long */
 	sc_init_one(&main_list, NANO_UNJUSTIFY_KEY, N_("UnJustify"),
 		IFHELP(NULL, NANO_NO_KEY), NANO_UNJUSTIFY_FKEY,
-		NANO_NO_KEY, NOVIEW, 0);
+		NANO_NO_KEY, NOVIEW, NULL);
     else
     /* Translators: try to keep this string under 10 characters long */
 	sc_init_one(&main_list, NANO_UNCUT_KEY, N_("UnCut Txt"),
@@ -550,12 +554,12 @@ void shortcut_init(bool unjustify)
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_NO_KEY, beg_of_par_msg,
 	IFHELP(nano_parabegin_msg, NANO_PARABEGIN_ALTKEY1), NANO_NO_KEY,
-	NANO_PARABEGIN_ALTKEY2, VIEW, do_para_begin);
+	NANO_PARABEGIN_ALTKEY2, VIEW, do_para_begin_void);
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_NO_KEY, end_of_par_msg,
 	IFHELP(nano_paraend_msg, NANO_PARAEND_ALTKEY1), NANO_NO_KEY,
-	NANO_PARAEND_ALTKEY2, VIEW, do_para_end);
+	NANO_PARAEND_ALTKEY2, VIEW, do_para_end_void);
 #endif
 
 #ifdef ENABLE_MULTIBUFFER
@@ -572,6 +576,13 @@ void shortcut_init(bool unjustify)
 	IFHELP(nano_verbatim_msg, NANO_VERBATIM_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, NOVIEW, do_verbatim_input);
 
+#ifndef NANO_SMALL
+    /* Translators: try to keep this string under 10 characters long */
+    sc_init_one(&main_list, NANO_NO_KEY, cut_till_end_msg,
+	IFHELP(nano_cut_till_end_msg, NANO_CUTTILLEND_ALTKEY),
+	NANO_NO_KEY, NANO_NO_KEY, NOVIEW, do_cut_till_end);
+#endif
+
 #ifndef DISABLE_JUSTIFY
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_NO_KEY, fulljstify_msg,
@@ -600,7 +611,7 @@ void shortcut_init(bool unjustify)
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_CANCEL_KEY, cancel_msg,
 	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_FIRSTLINE_KEY, first_line_msg,
@@ -615,47 +626,53 @@ void shortcut_init(bool unjustify)
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_TOOTHERSEARCH_KEY, replace_msg,
 	IFHELP(nano_replace_msg, NANO_NO_KEY), NANO_REPLACE_FKEY,
-	NANO_NO_KEY, VIEW, do_replace);
+	NANO_NO_KEY, VIEW, NULL);
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_TOGOTOLINE_KEY, go_to_line_msg,
 	IFHELP(nano_gotoline_msg, NANO_NO_KEY), NANO_GOTOLINE_FKEY,
-	NANO_NO_KEY, VIEW, do_gotoline_void);
+	NANO_NO_KEY, VIEW, NULL);
 
 #ifndef DISABLE_JUSTIFY
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_PARABEGIN_KEY, beg_of_par_msg,
 	IFHELP(nano_parabegin_msg, NANO_PARABEGIN_ALTKEY1), NANO_NO_KEY,
-	NANO_PARABEGIN_ALTKEY2, VIEW, do_para_begin);
+	NANO_PARABEGIN_ALTKEY2, VIEW, do_para_begin_void);
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_PARAEND_KEY, end_of_par_msg,
 	IFHELP(nano_paraend_msg, NANO_PARAEND_ALTKEY1), NANO_NO_KEY,
-	NANO_PARAEND_ALTKEY2, VIEW, do_para_end);
+	NANO_PARAEND_ALTKEY2, VIEW, do_para_end_void);
 #endif
 
 #ifndef NANO_SMALL
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_NO_KEY, case_sens_msg,
 	IFHELP(nano_case_msg, TOGGLE_CASE_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_NO_KEY, direction_msg,
 	IFHELP(nano_reverse_msg, TOGGLE_BACKWARDS_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
 #ifdef HAVE_REGEX_H
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_NO_KEY, regexp_msg,
 	IFHELP(nano_regexp_msg, TOGGLE_REGEXP_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 #endif
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_HISTORY_KEY, history_msg,
 	IFHELP(nano_editstr_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
+
+    /* Translators: try to keep this string under 10 characters long */
+    sc_init_one(&whereis_list, NANO_CUTTILLEND_KEY, cut_till_end_msg,
+	IFHELP(nano_cut_till_end_msg, NANO_CUTTILLEND_ALTKEY),
+	NANO_NO_KEY, NANO_NO_KEY, NOVIEW, do_cut_till_end);
+#endif /* !NANO_SMALL */
 
 #ifndef DISABLE_JUSTIFY
     /* Translators: try to keep this string under 10 characters long */
@@ -663,7 +680,6 @@ void shortcut_init(bool unjustify)
 	IFHELP(nano_fulljustify_msg, NANO_FULLJUSTIFY_ALTKEY),
 	NANO_NO_KEY, NANO_NO_KEY, NOVIEW, do_full_justify);
 #endif
-#endif /* !NANO_SMALL */
 
     free_shortcutage(&replace_list);
 
@@ -679,7 +695,7 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&replace_list, NANO_CANCEL_KEY, cancel_msg,
 	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&replace_list, NANO_FIRSTLINE_KEY, first_line_msg,
 	IFHELP(nano_firstline_msg, NANO_NO_KEY), NANO_FIRSTLINE_FKEY,
@@ -692,30 +708,30 @@ void shortcut_init(bool unjustify)
     /* Translators: try to keep this string under 12 characters long */
     sc_init_one(&replace_list, NANO_TOOTHERSEARCH_KEY, N_("No Replace"),
 	IFHELP(nano_whereis_msg, NANO_NO_KEY), NANO_REPLACE_FKEY,
-	NANO_NO_KEY, VIEW, do_search);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&replace_list, NANO_TOGOTOLINE_KEY, go_to_line_msg,
 	IFHELP(nano_gotoline_msg, NANO_NO_KEY), NANO_GOTOLINE_FKEY,
-	NANO_NO_KEY, VIEW, do_gotoline_void);
+	NANO_NO_KEY, VIEW, NULL);
 
 #ifndef NANO_SMALL
     sc_init_one(&replace_list, NANO_NO_KEY, case_sens_msg,
 	IFHELP(nano_case_msg, TOGGLE_CASE_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&replace_list, NANO_NO_KEY, direction_msg,
 	IFHELP(nano_reverse_msg, TOGGLE_BACKWARDS_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
 #ifdef HAVE_REGEX_H
     sc_init_one(&replace_list, NANO_NO_KEY, regexp_msg,
 	IFHELP(nano_regexp_msg, TOGGLE_REGEXP_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 #endif
 
     sc_init_one(&replace_list, NANO_HISTORY_KEY, history_msg,
 	IFHELP(nano_editstr_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 #endif /* !NANO_SMALL */
 
     free_shortcutage(&replace_list_2);
@@ -732,7 +748,7 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&replace_list_2, NANO_CANCEL_KEY, cancel_msg,
 	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&replace_list_2, NANO_FIRSTLINE_KEY, first_line_msg,
 	IFHELP(nano_firstline_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -745,7 +761,7 @@ void shortcut_init(bool unjustify)
 #ifndef NANO_SMALL
     sc_init_one(&replace_list_2, NANO_HISTORY_KEY, history_msg,
 	IFHELP(nano_editstr_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 #endif
 
     free_shortcutage(&gotoline_list);
@@ -762,7 +778,7 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&gotoline_list, NANO_CANCEL_KEY, cancel_msg,
 	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&gotoline_list, NANO_FIRSTLINE_KEY, first_line_msg,
 	IFHELP(nano_firstline_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -774,30 +790,34 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&gotoline_list, NANO_TOOTHERWHEREIS_KEY,
 	N_("Go To Text"), IFHELP(nano_whereis_msg, NANO_NO_KEY),
-	NANO_NO_KEY, NANO_NO_KEY, VIEW, do_search);
+	NANO_NO_KEY, NANO_NO_KEY, VIEW, NULL);
 
 #ifndef DISABLE_HELP
     free_shortcutage(&help_list);
 
     sc_init_one(&help_list, NANO_PREVPAGE_KEY, prev_page_msg,
 	IFHELP(nano_prevpage_msg, NANO_NO_KEY), NANO_PREVPAGE_FKEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&help_list, NANO_NEXTPAGE_KEY, next_page_msg,
 	IFHELP(nano_nextpage_msg, NANO_NO_KEY), NANO_NEXTPAGE_FKEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&help_list, NANO_PREVLINE_KEY, N_("Prev Line"),
 	IFHELP(nano_prevline_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&help_list, NANO_NEXTLINE_KEY, N_("Next Line"),
 	IFHELP(nano_nextline_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
+
+    sc_init_one(&help_list, NANO_REFRESH_KEY, N_("Refresh"),
+	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY,
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&help_list, NANO_EXIT_KEY, exit_msg,
 	IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 #endif
 
     free_shortcutage(&writefile_list);
@@ -814,7 +834,7 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&writefile_list, NANO_CANCEL_KEY, cancel_msg,
 	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
 #ifndef DISABLE_BROWSER
     /* If we're using restricted mode, the file browser is disabled.
@@ -823,7 +843,7 @@ void shortcut_init(bool unjustify)
     if (!ISSET(RESTRICTED))
 	sc_init_one(&writefile_list, NANO_TOFILES_KEY, to_files_msg,
 		IFHELP(nano_tofiles_msg, NANO_NO_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, NOVIEW, 0);
+		NANO_NO_KEY, NOVIEW, NULL);
 #endif
 
 #ifndef NANO_SMALL
@@ -837,33 +857,33 @@ void shortcut_init(bool unjustify)
     if (!ISSET(RESTRICTED))
 	sc_init_one(&writefile_list, NANO_NO_KEY, N_("DOS Format"),
 		IFHELP(nano_dos_msg, TOGGLE_DOS_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, NOVIEW, 0);
+		NANO_NO_KEY, NOVIEW, NULL);
 
     /* Translators: try to keep this string under 16 characters long */
     if (!ISSET(RESTRICTED))
 	sc_init_one(&writefile_list, NANO_NO_KEY, N_("Mac Format"),
 		IFHELP(nano_mac_msg, TOGGLE_MAC_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, NOVIEW, 0);
+		NANO_NO_KEY, NOVIEW, NULL);
 #endif
 
     /* Translators: try to keep this string under 16 characters long */
     if (!ISSET(RESTRICTED))
 	sc_init_one(&writefile_list, NANO_NO_KEY, N_("Append"),
 		IFHELP(nano_append_msg, NANO_APPEND_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, NOVIEW, 0);
+		NANO_NO_KEY, NOVIEW, NULL);
 
     /* Translators: try to keep this string under 16 characters long */
     if (!ISSET(RESTRICTED))
 	sc_init_one(&writefile_list, NANO_NO_KEY, N_("Prepend"),
 		IFHELP(nano_prepend_msg, NANO_PREPEND_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, NOVIEW, 0);
+		NANO_NO_KEY, NOVIEW, NULL);
 
 #ifndef NANO_SMALL
     /* Translators: try to keep this string under 16 characters long */
     if (!ISSET(RESTRICTED))
 	sc_init_one(&writefile_list, NANO_NO_KEY, N_("Backup File"),
 		IFHELP(nano_backup_msg, TOGGLE_BACKUP_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, NOVIEW, 0);
+		NANO_NO_KEY, NOVIEW, NULL);
 #endif
 
     free_shortcutage(&insertfile_list);
@@ -880,7 +900,7 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&insertfile_list, NANO_CANCEL_KEY, cancel_msg,
 		IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, VIEW, 0);
+		NANO_NO_KEY, VIEW, NULL);
 
 #ifndef DISABLE_BROWSER
     /* If we're using restricted mode, the file browser is disabled.
@@ -888,7 +908,7 @@ void shortcut_init(bool unjustify)
     if (!ISSET(RESTRICTED))
 	sc_init_one(&insertfile_list, NANO_TOFILES_KEY, to_files_msg,
 		IFHELP(nano_tofiles_msg, NANO_NO_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, NOVIEW, 0);
+		NANO_NO_KEY, NOVIEW, NULL);
 #endif
 
 #ifndef NANO_SMALL
@@ -898,7 +918,7 @@ void shortcut_init(bool unjustify)
     if (!ISSET(RESTRICTED))
 	sc_init_one(&insertfile_list, NANO_TOOTHERINSERT_KEY,
 		N_("Execute Command"), IFHELP(nano_execute_msg,
-		NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY, NOVIEW, 0);
+		NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY, NOVIEW, NULL);
 
 #ifdef ENABLE_MULTIBUFFER
     /* If we're using restricted mode, the multibuffer toggle is
@@ -907,7 +927,7 @@ void shortcut_init(bool unjustify)
     if (!ISSET(RESTRICTED))
 	sc_init_one(&insertfile_list, NANO_NO_KEY, new_buffer_msg,
 		IFHELP(nano_multibuffer_msg, TOGGLE_MULTIBUFFER_KEY),
-		NANO_NO_KEY, NANO_NO_KEY, NOVIEW, 0);
+		NANO_NO_KEY, NANO_NO_KEY, NOVIEW, NULL);
 #endif
 #endif
 
@@ -926,7 +946,7 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&spell_list, NANO_CANCEL_KEY, cancel_msg,
 	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 #endif
 
 #ifndef NANO_SMALL
@@ -944,16 +964,16 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&extcmd_list, NANO_CANCEL_KEY, cancel_msg,
 	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&extcmd_list, NANO_TOOTHERINSERT_KEY, N_("Insert File"),
 	IFHELP(nano_insert_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
 #ifdef ENABLE_MULTIBUFFER
     sc_init_one(&extcmd_list, NANO_NO_KEY, new_buffer_msg,
 	IFHELP(nano_multibuffer_msg, TOGGLE_MULTIBUFFER_KEY),
-	NANO_NO_KEY, NANO_NO_KEY, NOVIEW, 0);
+	NANO_NO_KEY, NANO_NO_KEY, NOVIEW, NULL);
 #endif
 #endif
 
@@ -972,20 +992,24 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&browser_list, NANO_EXIT_KEY, exit_msg,
 	IFHELP(nano_exitbrowser_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&browser_list, NANO_PREVPAGE_KEY, prev_page_msg,
 	IFHELP(nano_prevpage_msg, NANO_NO_KEY), NANO_PREVPAGE_FKEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 
     sc_init_one(&browser_list, NANO_NEXTPAGE_KEY, next_page_msg,
 	IFHELP(nano_nextpage_msg, NANO_NO_KEY), NANO_NEXTPAGE_FKEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
+
+    sc_init_one(&browser_list, NANO_REFRESH_KEY, N_("Refresh"),
+	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY,
+	NANO_NO_KEY, VIEW, NULL);
 
     /* Translators: try to keep this string under 22 characters long */
     sc_init_one(&browser_list, NANO_GOTOLINE_KEY, N_("Go To Dir"),
 	IFHELP(nano_gotodir_msg, NANO_GOTOLINE_ALTKEY),
-	NANO_GOTOLINE_FKEY, NANO_NO_KEY, VIEW, 0);
+	NANO_GOTOLINE_FKEY, NANO_NO_KEY, VIEW, NULL);
 
     free_shortcutage(&gotodir_list);
 
@@ -1001,12 +1025,11 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&gotodir_list, NANO_CANCEL_KEY, cancel_msg,
 	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, 0);
+	NANO_NO_KEY, VIEW, NULL);
 #endif
 
-#if !defined(DISABLE_BROWSER) || !defined(DISABLE_HELP) || !defined(DISABLE_MOUSE)
     currshortcut = main_list;
-#endif
+
 #ifndef NANO_SMALL
     toggle_init();
 #endif
@@ -1058,12 +1081,13 @@ void toggle_init(void)
     /* If we're using restricted mode, the multibuffer toggle is
      * disabled.  It's useless since inserting files is disabled. */
     if (!ISSET(RESTRICTED))
-	toggle_init_one(TOGGLE_MULTIBUFFER_KEY, N_("Multiple file buffers"),
-		MULTIBUFFER);
+	toggle_init_one(TOGGLE_MULTIBUFFER_KEY,
+		N_("Multiple file buffers"), MULTIBUFFER);
 #endif
     toggle_init_one(TOGGLE_CONST_KEY, N_("Constant cursor position"),
 	CONSTUPDATE);
-    toggle_init_one(TOGGLE_AUTOINDENT_KEY, N_("Auto indent"), AUTOINDENT);
+    toggle_init_one(TOGGLE_AUTOINDENT_KEY, N_("Auto indent"),
+	AUTOINDENT);
 #ifndef DISABLE_WRAPPING
     toggle_init_one(TOGGLE_WRAP_KEY, N_("Auto line wrap"), NO_WRAP);
 #endif
@@ -1075,25 +1099,30 @@ void toggle_init(void)
 #ifndef DISABLE_MOUSE
     toggle_init_one(TOGGLE_MOUSE_KEY, N_("Mouse support"), USE_MOUSE);
 #endif
-    /* If we're using restricted mode, the no-conversion and backup
-     * backup toggles are disabled.  The former is useless since
-     * inserting files is disabled, and the latter is useless since
-     * backups are disabled. */
-    if (!ISSET(RESTRICTED)) {
+    /* If we're using restricted mode, the DOS/Mac conversion toggle is
+     * disabled.  It's useless since inserting files is disabled. */
+    if (!ISSET(RESTRICTED))
 	toggle_init_one(TOGGLE_NOCONVERT_KEY,
 		N_("No conversion from DOS/Mac format"), NO_CONVERT);
-	toggle_init_one(TOGGLE_BACKUP_KEY, N_("Backup files"), BACKUP_FILE);
-    }
-    toggle_init_one(TOGGLE_SMOOTH_KEY, N_("Smooth scrolling"), SMOOTHSCROLL);
-    toggle_init_one(TOGGLE_SMARTHOME_KEY, N_("Smart home key"), SMART_HOME);
+    /* If we're using restricted mode, the backup toggle is disabled.
+     * It's useless since backups are disabled. */
+    if (!ISSET(RESTRICTED))
+	toggle_init_one(TOGGLE_BACKUP_KEY, N_("Backup files"),
+		BACKUP_FILE);
+    toggle_init_one(TOGGLE_SMOOTH_KEY, N_("Smooth scrolling"),
+	SMOOTHSCROLL);
+    toggle_init_one(TOGGLE_SMARTHOME_KEY, N_("Smart home key"),
+	SMART_HOME);
 #ifdef ENABLE_COLOR
     toggle_init_one(TOGGLE_SYNTAX_KEY, N_("Color syntax highlighting"),
-	COLOR_SYNTAX);
+	NO_COLOR_SYNTAX);
 #endif
 #ifdef ENABLE_NANORC
     toggle_init_one(TOGGLE_WHITESPACE_KEY, N_("Whitespace display"),
 	WHITESPACE_DISPLAY);
 #endif
+    toggle_init_one(TOGGLE_MORESPACE_KEY, N_("Use of more space for editing"),
+	MORE_SPACE);
 }
 
 #ifdef DEBUG
@@ -1160,7 +1189,10 @@ void thanks_for_all_the_fish(void)
 	free(answer);
     if (cutbuffer != NULL)
 	free_filestruct(cutbuffer);
-
+#ifndef DISABLE_JUSTIFY
+    if (jusbuffer != NULL)
+	free_filestruct(jusbuffer);
+#endif
     free_shortcutage(&main_list);
     free_shortcutage(&whereis_list);
     free_shortcutage(&replace_list);
