@@ -1,8 +1,8 @@
-/* $Id: nano.h,v 1.66 2004/11/07 23:05:48 dolorous Exp $ */
+/* $Id: nano.h,v 1.85 2005/03/20 07:24:49 dolorous Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
- *   Copyright (C) 1999-2004 Chris Allegretta                             *
+ *   Copyright (C) 1999-2005 Chris Allegretta                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -44,8 +44,10 @@
 #define charalloc(howmuch) (char *)nmalloc((howmuch) * sizeof(char))
 #define charealloc(ptr, howmuch) (char *)nrealloc(ptr, (howmuch) * sizeof(char))
 #define charmove(dest, src, n) memmove(dest, src, (n) * sizeof(char))
+#define charcpy(dest, src, n) memcpy(dest, src, (n) * sizeof(char))
 
 #ifdef BROKEN_REGEXEC
+#undef regexec
 #define regexec(preg, string, nmatch, pmatch, eflags) regexec_safe(preg, string, nmatch, pmatch, eflags)
 #endif
 
@@ -69,60 +71,52 @@
 #endif /* CURSES_H */
 
 #ifdef ENABLE_NLS
-#  ifdef HAVE_LIBINTL_H
-#    include <libintl.h>
-#  endif
-#  define _(string) gettext(string)
-#  define P_(singular, plural, number) ngettext(singular, plural, number)
+#ifdef HAVE_LIBINTL_H
+#include <libintl.h>
+#endif
+#define _(string) gettext(string)
+#define P_(singular, plural, number) ngettext(singular, plural, number)
 #else
-#  define _(string) (string)
-#  define P_(singular, plural, number) (number == 1 ? singular : plural)
+#define _(string) (string)
+#define P_(singular, plural, number) (number == 1 ? singular : plural)
 #endif
 #define gettext_noop(string) (string)
 #define N_(string) gettext_noop(string)
 	/* Mark a string that will be sent to gettext later. */
 
+#include <stddef.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "config.h"
 
-/* If no snprintf()/vsnprintf(), use the versions from glib. */
+/* If no snprintf() or vsnprintf(), use the versions from glib. */
 #if !defined(HAVE_SNPRINTF) || !defined(HAVE_VSNPRINTF)
 #include <glib.h>
-# ifndef HAVE_SNPRINTF
-#  define snprintf g_snprintf
-# endif
-# ifndef HAVE_VSNPRINTF
-#  define vsnprintf g_vsnprintf
-# endif
+#ifndef HAVE_SNPRINTF
+#define snprintf g_snprintf
+#endif
+#ifndef HAVE_VSNPRINTF
+#define vsnprintf g_vsnprintf
 #endif
-
-/* If no isblank(), strcasecmp(), strncasecmp(), strcasestr(),
- * strnlen(), getdelim(), or getline(), use the versions we have. */
-#ifndef HAVE_ISBLANK
-#define isblank is_blank_char
 #endif
 
+/* If no strcasecmp(), strncasecmp(), strcasestr(), strnlen(),
+ * getdelim(), or getline(), use the versions we have. */
 #ifndef HAVE_STRCASECMP
-#define strcasecmp nstricmp
+#define strcasecmp nstrcasecmp
 #endif
-
 #ifndef HAVE_STRNCASECMP
-#define strncasecmp nstrnicmp
+#define strncasecmp nstrncasecmp
 #endif
-
 #ifndef HAVE_STRCASESTR
-#define strcasestr nstristr
+#define strcasestr nstrcasestr
 #endif
-
 #ifndef HAVE_STRNLEN
 #define strnlen nstrnlen
 #endif
-
 #ifndef HAVE_GETDELIM
 #define getdelim ngetdelim
 #endif
-
 #ifndef HAVE_GETLINE
 #define getline ngetline
 #endif
@@ -145,10 +139,6 @@
 #endif
 
 /* Enumeration types. */
-typedef enum {
-    CUT_LINE, CUT_MARKED, CUT_TO_EOL
-} cut_type;
-
 typedef enum {
     NIX_FILE, DOS_FILE, MAC_FILE
 } file_format;
@@ -161,10 +151,6 @@ typedef enum {
     TOP, CENTER, NONE
 } topmidnone;
 
-typedef enum {
-    NO_SEQ, ESCAPE_SEQ, UTF8_SEQ
-} seq_type;
-
 /* Structure types. */
 typedef struct filestruct {
     char *data;
@@ -179,30 +165,31 @@ typedef struct openfilestruct {
 #ifndef NANO_SMALL
     struct stat originalfilestat;
 #endif
-    struct openfilestruct *next;	/* Next node. */
-    struct openfilestruct *prev;	/* Previous node. */
-    struct filestruct *fileage;	/* Current file. */
-    struct filestruct *filebot;	/* Current file's last line. */
+    struct openfilestruct *next;
+				/* Next node. */
+    struct openfilestruct *prev;
+				/* Previous node. */
+    filestruct *fileage;	/* Current file. */
+    filestruct *filebot;	/* Current file's last line. */
+    filestruct *edittop;	/* Current top of edit window. */
+    filestruct *current;	/* Current file's line. */
 #ifndef NANO_SMALL
-    struct filestruct *file_mark_beginbuf;
+    filestruct *mark_beginbuf;
 				/* Current file's beginning marked
 				 * line. */
-    size_t file_mark_beginx;	/* Current file's beginning marked
+    size_t mark_beginx;		/* Current file's beginning marked
 				 * line's x-coordinate position. */
 #endif
-    size_t file_current_x;	/* Current file's x-coordinate
+    size_t current_x;		/* Current file's x-coordinate
 				 * position. */
-    int file_current_y;		/* Current file's y-coordinate
-				 * position. */
-    size_t file_placewewant;	/* Current file's place we want. */
-    int file_totlines;		/* Current file's total number of
+    size_t placewewant;		/* Current file's place we want. */
+    int totlines;		/* Current file's total number of
 				 * lines. */
-    long file_totsize;		/* Current file's total size. */
-    int file_lineno;		/* Current file's line number. */
-    long file_flags;		/* Current file's flags: modification
+    size_t totsize;		/* Current file's total size. */
+    unsigned long flags;	/* Current file's flags: modification
 				 * status (and marking status, if
 				 * available). */
-    file_format file_fmt;	/* Current file's format. */
+    file_format fmt;		/* Current file's format. */
 } openfilestruct;
 #endif
 
@@ -222,7 +209,7 @@ typedef struct shortcut {
     int metaval;	/* Meta key we want bound. */
     int funcval;	/* Function key we want bound. */
     int miscval;	/* Other Meta key we want bound. */
-    int viewok;		/* Is this function legal in view mode? */
+    bool viewok;	/* Is this function legal in view mode? */
     void (*func)(void);	/* Function to call when we catch this key. */
     const char *desc;	/* Description, e.g. "Page Up". */
 #ifndef DISABLE_HELP
@@ -317,13 +304,15 @@ typedef struct historyheadtype {
 #define NO_CONVERT		(1<<19)
 #define BACKUP_FILE		(1<<20)
 #define NO_RCFILE		(1<<21)
-#define COLOR_SYNTAX		(1<<22)
+#define NO_COLOR_SYNTAX		(1<<22)
 #define PRESERVE		(1<<23)
 #define HISTORY_CHANGED		(1<<24)
 #define HISTORYLOG		(1<<25)
 #define RESTRICTED		(1<<26)
 #define SMART_HOME		(1<<27)
 #define WHITESPACE_DISPLAY	(1<<28)
+#define MORE_SPACE		(1<<29)
+#define NO_UTF8			(1<<30)
 
 /* Control key sequences, changing these would be very very bad. */
 #define NANO_CONTROL_SPACE 0
@@ -477,6 +466,8 @@ typedef struct historyheadtype {
 #define NANO_BRACKET_KEY	NANO_ALT_RBRACKET
 #define NANO_NEXTWORD_KEY	NANO_CONTROL_SPACE
 #define NANO_PREVWORD_KEY	NANO_ALT_SPACE
+#define NANO_CUTTILLEND_KEY	NANO_CONTROL_X
+#define NANO_CUTTILLEND_ALTKEY	NANO_ALT_T
 #define NANO_PARABEGIN_KEY	NANO_CONTROL_W
 #define NANO_PARABEGIN_ALTKEY1	NANO_ALT_LPAREN
 #define NANO_PARABEGIN_ALTKEY2	NANO_ALT_9
@@ -508,6 +499,7 @@ typedef struct historyheadtype {
 #define TOGGLE_SYNTAX_KEY	NANO_ALT_Y
 #define TOGGLE_SMARTHOME_KEY	NANO_ALT_H
 #define TOGGLE_WHITESPACE_KEY	NANO_ALT_P
+#define TOGGLE_MORESPACE_KEY	NANO_ALT_O
 #endif /* !NANO_SMALL */
 
 #define MAIN_VISIBLE 12
@@ -516,10 +508,7 @@ typedef struct historyheadtype {
 #define NOVIEW FALSE
 
 /* Minimum editor window rows required for nano to work correctly. */
-#define MIN_EDITOR_ROWS 3
-
-/* Minimum editor window cols required for nano to work correctly. */
-#define MIN_EDITOR_COLS 10
+#define MIN_EDITOR_ROWS 1
 
 /* Default number of characters from end-of-line where text wrapping
  * occurs. */
