@@ -1,8 +1,8 @@
-/* $Id: utils.c,v 1.40 2004/11/22 16:04:18 dolorous Exp $ */
+/* $Id: utils.c,v 1.65 2005/02/22 23:22:37 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
- *   Copyright (C) 1999-2004 Chris Allegretta                             *
+ *   Copyright (C) 1999-2005 Chris Allegretta                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -27,6 +27,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <unistd.h>
+#include <pwd.h>
 #include <ctype.h>
 #include <errno.h>
 #include <assert.h>
@@ -35,7 +36,6 @@
 
 #ifdef HAVE_REGEX_H
 #ifdef BROKEN_REGEXEC
-#undef regexec
 int regexec_safe(const regex_t *preg, const char *string, size_t nmatch,
 	regmatch_t pmatch[], int eflags)
 {
@@ -43,8 +43,7 @@ int regexec_safe(const regex_t *preg, const char *string, size_t nmatch,
 	return regexec(preg, string, nmatch, pmatch, eflags);
     return REG_NOMATCH;
 }
-#define regexec(preg, string, nmatch, pmatch, eflags) regexec_safe(preg, string, nmatch, pmatch, eflags)
-#endif /* BROKEN_REGEXEC */
+#endif
 
 int regexp_bol_or_eol(const regex_t *preg, const char *string)
 {
@@ -54,22 +53,6 @@ int regexp_bol_or_eol(const regex_t *preg, const char *string)
 }
 #endif /* HAVE_REGEX_H */
 
-#ifndef HAVE_ISBLANK
-/* This function is equivalent to isblank(). */
-int is_blank_char(int c)
-{
-    return (c == '\t' || c == ' ');
-}
-#endif
-
-/* This function is equivalent to iscntrl(), except in that it also
- * handles control characters with their high bits set. */
-int is_cntrl_char(int c)
-{
-    return (-128 <= c && c < -96) || (0 <= c && c < 32) ||
-	(127 <= c && c < 160);
-}
-
 int num_of_digits(int n)
 {
     int i = 1;
@@ -85,6 +68,23 @@ int num_of_digits(int n)
     return i;
 }
 
+/* Return the user's home directory.  We use $HOME, and if that fails,
+ * we fall back on getpwuid(). */
+void get_homedir(void)
+{
+    if (homedir == NULL) {
+	const char *homenv = getenv("HOME");
+
+	if (homenv == NULL) {
+	    const struct passwd *userage = getpwuid(geteuid());
+
+	    if (userage != NULL)
+		homenv = userage->pw_dir;
+	}
+	homedir = mallocstrcpy(NULL, homenv);
+    }
+}
+
 /* Read a ssize_t from str, and store it in *val (if val is not NULL).
  * On error, we return FALSE and don't change *val.  Otherwise, we
  * return TRUE. */
@@ -94,11 +94,15 @@ bool parse_num(const char *str, ssize_t *val)
     ssize_t j;
 
     assert(str != NULL);
+
     j = (ssize_t)strtol(str, &first_error, 10);
+
     if (errno == ERANGE || *str == '\0' || *first_error != '\0')
 	return FALSE;
+
     if (val != NULL)
 	*val = j;
+
     return TRUE;
 }
 
@@ -140,107 +144,6 @@ void sunder(char *str)
     }
 }
 
-#ifndef HAVE_STRCASECMP
-/* This function is equivalent to strcasecmp(). */
-int nstricmp(const char *s1, const char *s2)
-{
-    assert(s1 != NULL && s2 != NULL);
-
-    for (; *s1 != '\0' && *s2 != '\0'; s1++, s2++) {
-	if (tolower(*s1) != tolower(*s2))
-	    break;
-    }
-
-    return (tolower(*s1) - tolower(*s2));
-}
-#endif
-
-#ifndef HAVE_STRNCASECMP
-/* This function is equivalent to strncasecmp(). */
-int nstrnicmp(const char *s1, const char *s2, size_t n)
-{
-    assert(s1 != NULL && s2 != NULL);
-
-    for (; n > 0 && *s1 != '\0' && *s2 != '\0'; n--, s1++, s2++) {
-	if (tolower(*s1) != tolower(*s2))
-	    break;
-    }
-
-    if (n > 0)
-	return (tolower(*s1) - tolower(*s2));
-    else
-	return 0;
-}
-#endif
-
-#ifndef HAVE_STRCASESTR
-/* This function is equivalent to strcasestr().  It was adapted from
- * mutt's mutt_stristr() function. */
-const char *nstristr(const char *haystack, const char *needle)
-{
-    assert(haystack != NULL && needle != NULL);
-
-    for (; *haystack != '\0'; haystack++) {
-	const char *p = haystack;
-	const char *q = needle;
-
-	for (; tolower(*p) == tolower(*q) && *q != '\0'; p++, q++)
-	    ;
-
-	if (*q == '\0')
-	    return haystack;
-    }
-
-    return NULL;
-}
-#endif
-
-/* None of this is needed if we're using NANO_SMALL! */
-#ifndef NANO_SMALL
-const char *revstrstr(const char *haystack, const char *needle, const
-	char *rev_start)
-{
-    for (; rev_start >= haystack; rev_start--) {
-	const char *r, *q;
-
-	for (r = rev_start, q = needle ; *q == *r && *q != '\0'; r++, q++)
-	    ;
-	if (*q == '\0')
-	    return rev_start;
-    }
-    return NULL;
-}
-
-const char *revstristr(const char *haystack, const char *needle, const
-	char *rev_start)
-{
-    for (; rev_start >= haystack; rev_start--) {
-	const char *r = rev_start, *q = needle;
-
-	for (; (tolower(*q) == tolower(*r)) && (*q != '\0') ; r++, q++)
-	    ;
-	if (*q == '\0')
-	    return rev_start;
-    }
-    return NULL;
-}
-#endif /* !NANO_SMALL */
-
-#ifndef HAVE_STRNLEN
-/* This function is equivalent to strnlen(). */
-size_t nstrnlen(const char *s, size_t maxlen)
-{
-    size_t n = 0;
-
-    assert(s != NULL);
-
-    for (; maxlen > 0 && *s != '\0'; maxlen--, n++, s++)
-	;
-
-    return n;
-}
-#endif
-
 #if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
 #ifndef HAVE_GETLINE
 /* This function is equivalent to getline().  It was adapted from
@@ -337,7 +240,7 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 	} else
 #endif /* !NANO_SMALL */
 	if (regexec(&search_regexp, start, 10, regmatches,
-		start > haystack ? REG_NOTBOL : 0) == 0) {
+		(start > haystack) ? REG_NOTBOL : 0) == 0) {
 	    const char *retval = start + regmatches[0].rm_so;
 
 	    regexec(&search_regexp, retval, 10, regmatches, 0);
@@ -346,7 +249,7 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 	return NULL;
     }
 #endif /* HAVE_REGEX_H */
-#if !defined(DISABLE_SPELLER) || !defined(NANO_SMALL)
+#if !defined(NANO_SMALL) || !defined(DISABLE_SPELLER)
     if (ISSET(CASE_SENSITIVE)) {
 #ifndef NANO_SMALL
 	if (ISSET(REVERSE_SEARCH))
@@ -358,9 +261,9 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 #endif /* !DISABLE_SPELLER || !NANO_SMALL */
 #ifndef NANO_SMALL
     else if (ISSET(REVERSE_SEARCH))
-	return revstristr(haystack, needle, start);
+	return mbrevstrcasestr(haystack, needle, start);
 #endif
-    return strcasestr(start, needle);
+    return mbstrcasestr(start, needle);
 }
 
 /* This is a wrapper for the perror() function.  The wrapper takes care
@@ -456,51 +359,6 @@ void remove_magicline(void)
     }
 }
 
-/* Calculate the number of lines and the number of characters between
- * begin and end, and return them in lines and size, respectively. */
-void get_totals(const filestruct *begin, const filestruct *end, int
-	*lines, long *size)
-{
-    const filestruct *f;
-
-    if (lines != NULL)
-	*lines = 0;
-    if (size != NULL)
-	*size = 0;
-
-    /* Go through the lines from begin to end->prev, if we can. */
-    for (f = begin; f != NULL && f != end; f = f->next) {
-	/* Count this line. */
-	if (lines != NULL)
-	    (*lines)++;
-
-	/* Count the number of characters on this line. */
-	if (size != NULL) {
-	    *size += strlen(f->data);
-
-	    /* Count the newline if we have one. */
-	    if (f->next != NULL)
-		(*size)++;
-	}
-    }
-
-    /* Go through the line at end, if we can. */
-    if (f != NULL) {
-	/* Count this line. */
-	if (lines != NULL)
-	    (*lines)++;
-
-	/* Count the number of characters on this line. */
-	if (size != NULL) {
-	    *size += strlen(f->data);
-
-	    /* Count the newline if we have one. */
-	    if (f->next != NULL)
-		(*size)++;
-	}
-    }
-}
-
 /* Set top_x and bot_x to the top and bottom x-coordinates of the mark,
  * respectively, based on the locations of top and bot.  If
  * right_side_up isn't NULL, set it to TRUE If the mark begins with
@@ -512,7 +370,7 @@ void mark_order(const filestruct **top, size_t *top_x, const filestruct
     assert(top != NULL && top_x != NULL && bot != NULL && bot_x != NULL);
 
     if ((current->lineno == mark_beginbuf->lineno && current_x >
-	 mark_beginx) || current->lineno > mark_beginbuf->lineno) {
+	mark_beginx) || current->lineno > mark_beginbuf->lineno) {
 	*top = mark_beginbuf;
 	*top_x = mark_beginx;
 	*bot = current;
@@ -530,104 +388,47 @@ void mark_order(const filestruct **top, size_t *top_x, const filestruct
 }
 #endif
 
-#ifndef DISABLE_TABCOMP
-/*
- * Routine to see if a text string is matched by a wildcard pattern.
- * Returns TRUE if the text is matched, or FALSE if it is not matched
- * or if the pattern is invalid.
- *  *		matches zero or more characters
- *  ?		matches a single character
- *  [abc]	matches 'a', 'b' or 'c'
- *  \c		quotes character c
- * Adapted from code written by Ingo Wilken, and
- * then taken from sash, Copyright (c) 1999 by David I. Bell
- * Permission is granted to use, distribute, or modify this source,
- * provided that this copyright notice remains intact.
- * Permission to distribute this code under the GPL has been granted.
- */
-int check_wildcard_match(const char *text, const char *pattern)
+/* Calculate the number of lines and the number of characters between
+ * begin and end, and return them in lines and size, respectively. */
+void get_totals(const filestruct *begin, const filestruct *end, int
+	*lines, size_t *size)
 {
-    const char *retrypat;
-    const char *retrytext;
-    int ch;
-    int found;
-    int len;
+    const filestruct *f;
 
-    retrypat = NULL;
-    retrytext = NULL;
+    if (lines != NULL)
+	*lines = 0;
+    if (size != NULL)
+	*size = 0;
 
-    while (*text != '\0' || *pattern != '\0') {
-	ch = *pattern++;
+    /* Go through the lines from begin to end->prev, if we can. */
+    for (f = begin; f != NULL && f != end; f = f->next) {
+	/* Count this line. */
+	if (lines != NULL)
+	    (*lines)++;
 
-	switch (ch) {
-	case '*':
-	    retrypat = pattern;
-	    retrytext = text;
-	    break;
+	/* Count the number of characters on this line. */
+	if (size != NULL) {
+	    *size += mbstrlen(f->data);
 
-	case '[':
-	    found = FALSE;
-
-	    while ((ch = *pattern++) != ']') {
-		if (ch == '\\')
-		    ch = *pattern++;
-
-		if (ch == '\0')
-		    return FALSE;
-
-		if (*text == ch)
-		    found = TRUE;
-	    }
-	    len = strlen(text);
-	    if (!found && len != 0) {
-		return FALSE;
-	    }
-	    if (found) {
-		if (strlen(pattern) == 0 && len == 1) {
-		    return TRUE;
-		}
-		if (len != 0) {
-		    text++;
-		    continue;
-		}
-	    }
-
-	    /* fall into next case */
-
-	case '?':
-	    if (*text++ == '\0')
-		return FALSE;
-
-	    break;
-
-	case '\\':
-	    ch = *pattern++;
-
-	    if (ch == '\0')
-		return FALSE;
-
-	    /* fall into next case */
-
-	default:
-	    if (*text == ch) {
-		if (*text != '\0')
-		    text++;
-		break;
-	    }
-
-	    if (*text != '\0') {
-		pattern = retrypat;
-		text = ++retrytext;
-		break;
-	    }
-
-	    return FALSE;
+	    /* Count the newline if we have one. */
+	    if (f->next != NULL)
+		(*size)++;
 	}
-
-	if (pattern == NULL)
-	    return FALSE;
     }
 
-    return TRUE;
+    /* Go through the line at end, if we can. */
+    if (f != NULL) {
+	/* Count this line. */
+	if (lines != NULL)
+	    (*lines)++;
+
+	/* Count the number of characters on this line. */
+	if (size != NULL) {
+	    *size += mbstrlen(f->data);
+
+	    /* Count the newline if we have one. */
+	    if (f->next != NULL)
+		(*size)++;
+	}
+    }
 }
-#endif
